package exchangeRateTable2;

import java.math.BigDecimal;
import java.util.Currency;
import java.util.Hashtable;
import java.util.Map;

/* Původní řešení třídy ExchangeRateTable má tyto nedostatky:
 * 
 * 1) Metoda convert řeší dvě věci (zjištění kurzu a výpočet konverze) a měla by
 *    se rozdělit na dvě samostatné metody - jednu pro zjištění kurzu a druhou
 *    pro vlastní konverzi.
 *
 * 2) Třida ExchangeTable řeší dvě věci (evidenci kurzů a provádění konverzí) a
      měla by se rozdělit do dvou tříd - jednu pro evidenci kurzů a druhou pro
 *    provádní transformací.
 *
 * 3) Kontrakt třídy pro evidenci kurzů by měl být oddělen od implementace.
 *    Takže by se mělo zavést rozhraní ExchangeRateTable a implementace by měla
 *    být přesunuta do třídy ExchangeRateTableIml. Toto řešení usnadní jednotkové
 *    testování a umožní v případě potřeby vytvořit jinou implementaci kurzovní
 *    tabulky (např. ukládající kurzy do databáze)
 *
 * 4) Pro reprezentaci měny by neměl být použit typ String, ale specializovaný typ.
 *    Pokud v podobném případě neexistuje vhodná třída v Java Core API, pak si 
 *    takovou třídu vytvoříme, pokud vhodná třída existuje, použijeme ji.
 *
 * 5) Pro reprezentaci kurzu i částky je nutné použít datový typ s pevnou
 ̇*    řádovou čárkou.
 */

/**
 * This interface represents exchange rate table.
 *
 * @author Petr Adamek
 */
interface ExchangeRateTable {

    public void setExchangeRate(Currency currency, BigDecimal exchangeRate);

    public BigDecimal getExchangeRate(Currency currency);

}

class ExchangeRateTableImpl implements ExchangeRateTable {

    private Map<Currency,BigDecimal> exchangeRates = new Hashtable<Currency, BigDecimal>();

    public void setExchangeRate(Currency currency, BigDecimal exchangeRate) {
        if (exchangeRate.signum() <= 0) {
            throw new IllegalArgumentException("Exchange rate must be grater than zero");
        }
        exchangeRates.put(currency, exchangeRate);
    }

    public BigDecimal getExchangeRate(Currency currency) {
        return exchangeRates.get(currency);
    }
}

class CurrencyConvertor {

    private ExchangeRateTable exchangeRateTable;

    public CurrencyConvertor(ExchangeRateTable exchangeRateTable) {
        this.exchangeRateTable = exchangeRateTable;
    }

    public BigDecimal convert(Currency targetCurrency, BigDecimal amount) {
        BigDecimal exchangeRate = exchangeRateTable.getExchangeRate(targetCurrency);
        return exchangeRate.multiply(amount);
    }
}