package searchEngine2;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/* Původní řešení třídy SearchEngine má tyto nedostatky:
 *
 * 1) Metody addText a search mají na prvním řádku stejný kód, který by měl
 *    být refaktorován do samostatné metody.
 *
 * 2) Metoda addString je docela složitá a její čitelnosti by prospělo, kdyby se
 *    její část extrahovala do samostatné metody.
 *
 * 3) Kontrakt třídy SearchEngine by měl být oddělen od implementace.
 *    Takže by se mělo zavést rozhraní SearchEngine a implementace by měla
 *    být přesunuta do třídy SearchEngineIml. Toto řešení usnadní jednotkové
 *    testování a umožní v případě potřeby vytvořit jinou implementaci
 *    vyhledávacího stroje.
 *
 * 4) Pro rozšíření o možnost logování všech dotazů je lepší místo dědičnosti
 *    použít kombinaci agregace a delegování. Zatímco dědičnost je statická
 *    vazba (je definována v době psaní kódu a nedá se změnit), agregace je
 *    dynamická (může být v době běhu programu měněna), redukuje míru statických
 *    závislostí a nabízí větší flexibilitu. Např. je možné vyměnit třídu
 *    vyhledávacího stroje za jinou implementaci, aniž by bylo nutné měnit
 *    jediný řádek zdrojového kódu třídy LoggingSearchEngine.
 *
 *    Mimochodem, kombinace agregace, delegování a implementace společného
 *    rozhraní, která je použita v tomto příkladu, se označuje jako návrhový
 *    vzor dekorátor.
 *
 *    Jiným dobrým příkladem výhod principu delegování a je knihovna proudů v 
 *    balíku java.io. Všimněte si, že je možné jednotlivé implementace proudů
 *    poměrně flexibilně kombinovat a spojovat podle potřeby.
 *
 */

 /**
 * This interface represents simple search engine.
 *
 * @author Petr Adamek
 */
interface SearchEngine {

    public void addText(String text);
    public Set<String> search(String query);

}

class SearchEngineImpl implements SearchEngine {

    Map<String,Set<String>> strings = new HashMap<String, Set<String>>();
    
    private static <K,V> Set<V> getSetFromMap(Map<K,Set<V>> map, K key) {
        Set<V> result  = map.get(key);
        if (result == null) {
            result = new HashSet<V>();
            map.put(key, result);
        }
        return result;
    }

    private void addKeyword(String keyword, String text) {
        getSetFromMap(strings, keyword).add(text);
    }

    private String[] splitToKeywords(String text) {
        return text.toLowerCase().split("\\s+"); // \s = any white charactes
    }

    public void addText(String text) {
        String[] keywords = splitToKeywords(text);
        for (String keyword : keywords) {
            addKeyword(keyword, text);
        }
    }

    private Set<String> getTextSet(String keyword) {
        Set<String> result  = strings.get(keyword);
        if (result == null) {
            return Collections.emptySet();
        } else {
            return result;
        }
    }

    public Set<String> search(String query) {
        String[] keywords = splitToKeywords(query);
        if (keywords.length <= 0) {
            return Collections.emptySet();
        }
        Set<String> result = new HashSet<String>(strings.get(keywords[0]));
        // Pokud dotaz obsahuje vice slov, aplikuje se operator AND
        for (int i = 1; i < keywords.length; i++) {
            result.retainAll(getTextSet(keywords[i]));
        }
        return result;
    }
}

/**
 * This class extends SearchEngine with ability to log all queries. It is based
 * on Decorator design pattern.
 *
 * @author Petr Adamek
 */
class LoggingSearchEngine implements SearchEngine {

    private SearchEngine searchEngine;

    public LoggingSearchEngine(SearchEngine searchEngine) {
        this.searchEngine = searchEngine;
    }

    public void addText(String text) {
        searchEngine.addText(text);
    }

    public Set<String> search(String query) {
        System.err.println(query);
        return searchEngine.search(query);
    }
    
}

public class SearchEngine2 {
    public static void main(String[] args) {
        SearchEngine searchEngine = new LoggingSearchEngine(new SearchEngineImpl());
        searchEngine.addText("Prvni veta");
        searchEngine.addText("Druha veta");
        searchEngine.addText("Prvni   blbost  ");
        System.out.println(searchEngine.search("Prvni"));
        System.out.println(searchEngine.search("veta"));
        System.out.println(searchEngine.search("veta   druha"));
    }
}